﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Linq;

namespace ItemsSourceSwitch
{
    public class ItemSourceContainer<T> : IEnumerable<T>, INotifyCollectionChanged
    {
        #region Members
        private List<ObservableCollection<T>> _collectionsToSwitchThrough;
        private int _activeCollectionIndex; 
        #endregion //Members

        #region Constructor
        public ItemSourceContainer(int activeCollectionIndex, params ObservableCollection<T>[] collections)
        {
            this._collectionsToSwitchThrough = collections.ToList();
            this._activeCollectionIndex = activeCollectionIndex;
        }
        #endregion //Constructor

        #region Properties
        public int ActiveCollectionIndex
        {
            get { return this._activeCollectionIndex; }
            set
            {
                if (this._activeCollectionIndex != value)
                {
                    this.UnHookHadlers(this._collectionsToSwitchThrough[this._activeCollectionIndex]);

                    this._activeCollectionIndex = value;

                    var collection = this._collectionsToSwitchThrough[this._activeCollectionIndex];

                    //Fire an Reset to notify XamGrid to refresh
                    this.OnCollectionChanged(new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Reset));

                    this.HookUpHadlers(collection);
                }
            }
        }
        #endregion //Properties

        private void HookUpHadlers(ObservableCollection<T> targetCollection)
        {
            targetCollection.CollectionChanged += TargetCollection_CollectionChanged;
        }

        private void UnHookHadlers(ObservableCollection<T> targetCollection)
        {
            targetCollection.CollectionChanged -= TargetCollection_CollectionChanged;
        }

        void TargetCollection_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            this.OnCollectionChanged(e);
        }

        #region IEnumerable implementation
        public IEnumerator<T> GetEnumerator()
        {
            return this._collectionsToSwitchThrough[_activeCollectionIndex].GetEnumerator();
        }

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return this._collectionsToSwitchThrough[_activeCollectionIndex].GetEnumerator();
        }
        #endregion //IEnumerable implementation

        #region INotifyCollectionChanged implementation
        protected void OnCollectionChanged(NotifyCollectionChangedEventArgs args)
        {
            var handler = this.CollectionChanged;
            if (handler != null)
            {
                handler(this, args);
            }
        }

        public event NotifyCollectionChangedEventHandler CollectionChanged;
        #endregion //INotifyCollectionChanged implementation
    }
}
